package controllers.workspaceApi.search

import controllers.workspaceApi.search.SearchApiModel.Facets
import org.silkframework.dataset.DatasetSpec.GenericDatasetSpec
import org.silkframework.rule.TransformSpec
import org.silkframework.runtime.activity.UserContext
import org.silkframework.workspace.ProjectTask

/**
  * Facet collector for the Transformation task.
  */
case class TransformFacetCollector() extends ItemTypeFacetCollector[ProjectTask[TransformSpec]] {
  override val facetCollectors: Seq[FacetCollector[ProjectTask[TransformSpec]]] = Seq(
    TransformInputResourceFacetCollector()
  )
}

/** Collect file resources from input datasets. */
case class TransformInputResourceFacetCollector() extends NoLabelKeywordFacetCollector[ProjectTask[TransformSpec]] {
  private val fileFacetCollector = DatasetFileFacetCollector()

  override def extractKeywordIds(projectTask: ProjectTask[TransformSpec])
                                (implicit user: UserContext): Set[String] = {
    val inputTaskId = projectTask.data.selection.inputId
    projectTask.project.taskOption[GenericDatasetSpec](inputTaskId).toSet flatMap { projectDatasetSpec: ProjectTask[GenericDatasetSpec] =>
      fileFacetCollector.extractKeywordIds(projectDatasetSpec)
    }
  }

  override def appliesForFacet: SearchApiModel.Facet = Facets.transformInputResource
}